@@ -2,7 +2,9 @@
    this repository contains the full copyright notices and license terms. */
 
 /* eslint-disable no-redeclare */
-var Sao = {};
+var Sao = {
+    __version__: '7.0.18',
+};
 /* eslint-enable no-redeclare */
 
 (function() {
@@ -98,9 +100,9 @@ var Sao = {};
             // Implementation is not strictly equivalent but works for most
             // cases
             var result = [];
-            value.forEach(function(e) {
+            for (const e of value) {
                 result.push(e);
-            });
+            }
             return result;
         };
     }
@@ -178,6 +180,47 @@ var Sao = {};
         return ClassConstructor;
     };
 
+    Sao.Logging = Sao.class_(Object, {
+        init: function() {
+            this.level = Sao.Logging.ERROR;
+        },
+        set_level: function(level) {
+            this.level = level;
+        },
+        _log: function(level, logger, args) {
+            if (this.level <= level) {
+                logger.apply(console, args);
+            }
+        },
+        debug: function() {
+            this._log(Sao.Logging.DEBUG, console.log, arguments);
+        },
+        info: function() {
+            this._log(Sao.Logging.INFO, console.info, arguments);
+        },
+        warn: function() {
+            this._log(Sao.Logging.WARNING, console.warn, arguments);
+        },
+        error: function() {
+            this._log(Sao.Logging.ERROR, console.error, arguments);
+        },
+        critical: function() {
+            this._log(Sao.Logging.CRITICAL, console.error, arguments);
+        },
+        assert: function() {
+            if (this.level <= Sao.Logging.DEBUG) {
+                console.assert.apply(console, arguments);
+            }
+        },
+    });
+    Sao.Logging.CRITICAL = 50;
+    Sao.Logging.ERROR = 40;
+    Sao.Logging.WARNING = 30;
+    Sao.Logging.INFO = 20;
+    Sao.Logging.DEBUG = 10;
+    Sao.Logging.NOTSET = 0;
+    Sao.Logger = new Sao.Logging();
+
     Sao.Decimal = Number;
 
     var _moment_to_string = moment.prototype.toString;
@@ -226,26 +269,15 @@ var Sao = {};
     Sao.Date.max.set({hour: 0, minute: 0, second: 0, millisecond: 0});
     Sao.Date.max.isDate = true;
 
-    Sao.DateTime = function(year, month, day, hour, minute, second,
-            millisecond, utc) {
+    Sao.DateTime = function(
+        year, month, day,
+        hour=0, minute=0, second=0, millisecond=0, utc=false) {
         var datetime;
         if (month === undefined) {
             datetime = moment(year);
             year = undefined;
         }
         else {
-            if (hour === undefined) {
-                hour = 0;
-            }
-            if (minute === undefined) {
-                minute = 0;
-            }
-            if (second === undefined) {
-                second = 0;
-            }
-            if (millisecond === undefined) {
-                millisecond = 0;
-            }
             datetime = moment();
         }
         if (utc) {
@@ -307,12 +339,20 @@ var Sao = {};
     Sao.config = {};
     Sao.config.limit = 1000;
     Sao.config.display_size = 20;
+    var doc_version = Sao.__version__.split('.').slice(0, 2);
+    if (parseInt(doc_version[1], 10) % 2) {
+        doc_version = 'latest';
+    } else {
+        doc_version = doc_version.join('.');
+    }
+    Sao.config.doc_url = `https://docs.tryton.org/${doc_version}`;
     Sao.config.bug_url = 'https://bugs.tryton.org/';
     Sao.config.title = 'Tryton';
     Sao.config.icon_colors = '#267f82,#3e4950,#e78e42'.split(',');
     Sao.config.calendar_colors = '#fff,#267f82'.split(',');
     Sao.config.graph_color = '#267f82';
     Sao.config.bus_timeout = 10 * 60 * 1000;
+    Sao.config.image_max_size = Math.pow(10, 6);
 
     Sao.i18n = i18n();
     Sao.i18n.setlang = function(lang) {
@@ -374,11 +414,12 @@ var Sao = {};
                 deferreds.push(Sao.common.MODELACCESS.load_models());
                 deferreds.push(Sao.common.ICONFACTORY.load_icons());
                 deferreds.push(Sao.common.MODELHISTORY.load_history());
+                deferreds.push(Sao.common.MODELNOTIFICATION.load_names());
                 deferreds.push(Sao.common.VIEW_SEARCH.load_searches());
                 return jQuery.when.apply(jQuery, deferreds).then(function() {
-                    (preferences.actions || []).forEach(function(action_id) {
+                    for (const action_id of (preferences.actions || [])) {
                         Sao.Action.execute(action_id, {}, null, {});
-                    });
+                    }
                     Sao.set_title();
                     var new_lang = preferences.language != Sao.i18n.getLocale();
                     var prm = jQuery.Deferred();
@@ -437,8 +478,7 @@ var Sao = {};
             }
         } else {
             url = decodeURIComponent(url);
-            for (var i = 0; i < Sao.Tab.tabs.length; i++) {
-                tab = Sao.Tab.tabs[i];
+            for (const tab of Sao.Tab.tabs) {
                 if (decodeURIComponent(tab.get_url()) == url) {
                     tab.show();
                     return;
@@ -459,12 +499,12 @@ var Sao = {};
         var path, params = {};
         if (i >= 0) {
             path = url.substring(0, i);
-            url.substring(i + 1).split('&').forEach(function(part) {
+            for (const part of url.substring(i + 1).split('&')) {
                 if (part) {
                     var item = part.split('=').map(decodeURIComponent);
                     params[item[0]] = item[1];
                 }
-            });
+            }
         } else {
             path = url;
         }
@@ -580,24 +620,46 @@ var Sao = {};
     Sao.login = function() {
         Sao.set_title();
         Sao.i18n.setlang().always(function() {
-            Sao.Session.get_credentials()
-                .then(function(session) {
-                    Sao.Session.current_session = session;
-                    return session.reload_context();
-                }).then(Sao.get_preferences).then(function(preferences) {
-                    Sao.menu(preferences);
-                    Sao.user_menu(preferences);
-                    Sao.open_url();
-                    Sao.Bus.listen();
+            Sao.Session.server_version()
+                .then(function(version) {
+                    if (JSON.stringify(version.split('.').slice(0, 2)) !==
+                        JSON.stringify(Sao.__version__.split('.').slice(0, 2)))
+                    {
+                        Sao.common.warning.run(
+                            Sao.i18n.gettext(
+                                "Incompatible version of the server."),
+                            Sao.i18n.gettext("Version mismatch"));
+                    } else {
+                        Sao.Session.get_credentials()
+                            .then(function(session) {
+                                Sao.Session.current_session = session;
+                                return session.reload_context();
+                            })
+                            .then(Sao.get_preferences)
+                            .then(function(preferences) {
+                                Sao.menu(preferences);
+                                Sao.user_menu(preferences);
+                                Sao.open_url();
+                                Sao.Bus.listen();
+                            });
+                    }
+                }, function() {
+                    Sao.common.warning.run(
+                        Sao.i18n.gettext(
+                            "Could not connect to the server."),
+                        Sao.i18n.gettext("Connection error"));
                 });
         });
     };
 
     Sao.logout = function() {
         var session = Sao.Session.current_session;
+        if (!session || !session.session) {
+            // Do not save states if there is no session
+            Sao.main_menu_screen = null;
+        }
         Sao.Tab.tabs.close(true).done(function() {
             jQuery('#user-preferences').empty();
-            jQuery('#user-favorites').empty();
             jQuery('#global-search').empty();
             jQuery('#menu').empty();
             session.do_logout().always(Sao.login);
@@ -606,9 +668,9 @@ var Sao = {};
     };
 
     Sao.preferences = function() {
-        Sao.Tab.tabs.close(true).done(function() {
+        return Sao.Tab.tabs.close(true).then(function() {
             jQuery('#user-preferences').empty();
-            jQuery('#user-favorites').empty();
+            jQuery('#global-search').empty();
             jQuery('#menu').empty();
             new Sao.Window.Preferences(function() {
                 Sao.Session.current_session.reset_context();
@@ -687,13 +749,13 @@ var Sao = {};
 
     Sao.user_menu = function(preferences) {
         jQuery('#user-preferences').empty();
-        jQuery('#user-favorites').empty();
         var user = jQuery('<a/>', {
             'href': '#',
             'title': preferences.status_bar,
         }).click(function(evt) {
             evt.preventDefault();
-            Sao.preferences();
+            user.prop('disabled', true);
+            Sao.preferences().then(() => user.prop('disabled', false));
         }).text(preferences.status_bar);
         jQuery('#user-preferences').append(user);
         if (preferences.avatar_badge_url) {
@@ -719,7 +781,7 @@ var Sao = {};
 
     Sao.main_menu_row_activate = function() {
         var screen = Sao.main_menu_screen;
-        var id = screen.get_id();
+        const id = screen.get_id();
         if (id) {
             // ids is not defined to prevent to add suffix
             Sao.Action.exec_keyword('tree_open', {
@@ -765,9 +827,10 @@ var Sao = {};
         Sao.Tab.tabs.splice(Sao.Tab.tabs.indexOf(form), 1);
         form.view_prm.done(function() {
             var view = form.screen.current_view;
-            view.table.removeClass('table table-bordered table-striped');
+            view.table.removeClass('table table-bordered');
             view.table.addClass('no-responsive');
             view.table.find('thead').hide();
+            view.table.find('colgroup > col.tree-menu').css('width', 0);
             var gs = new Sao.GlobalSearch();
             jQuery('#global-search').empty();
             jQuery('#global-search').append(gs.el);
@@ -805,7 +868,7 @@ var Sao = {};
             if (!cell) {
                 cell = this.get_cell();
             }
-            record.load(this.field.name).done(function() {
+            record.load(this.field.name).done(() => {
                 if (record._values.favorite !== null) {
                     var icon = 'tryton-star';
                     if (!record._values.favorite) {
@@ -813,15 +876,20 @@ var Sao = {};
                     }
                     cell.data('star', Boolean(record._values.favorite));
                     Sao.common.ICONFACTORY.get_icon_url(icon)
-                        .then(function(url) {
+                        .then(url => {
                             cell.attr('src', url);
                         });
                     cell.click({'record': record, 'button': cell},
                         this.favorite_click);
                     }
-                }.bind(this));
+                });
             return cell;
         },
+        set_visible: function() {
+        },
+        get_visible: function() {
+            return true;
+        },
         favorite_click: function(e) {
             // Prevent activate the action of the row
             e.stopImmediatePropagation();
@@ -852,11 +920,8 @@ var Sao = {};
     });
 
     Sao.Dialog = Sao.class_(Object, {
-        init: function(title, class_, size, keyboard) {
-            size = size || 'sm';
-            if (keyboard === undefined) {
-                keyboard = true;
-            }
+        init: function(
+            title, class_, size='sm', keyboard=true, small=null) {
             this.modal = jQuery('<div/>', {
                 'class': class_ + ' modal fade',
                 'role': 'dialog',
@@ -872,7 +937,7 @@ var Sao = {};
                 'class': 'modal-header'
             }).appendTo(this.content);
             if (title) {
-                this.add_title(title);
+                this.add_title(title, small);
             }
             this.body = jQuery('<div/>', {
                 'class': 'modal-body'
@@ -891,11 +956,15 @@ var Sao = {};
                 }
             });
         },
-        add_title: function(title) {
-            this.header.append(jQuery('<h4/>', {
+        add_title: function(title, small=null) {
+            var titleElement = jQuery('<h4/>', {
                 'class': 'modal-title',
                 'title': title,
-            }).text(Sao.common.ellipsize(title, 120)));
+            }).text(Sao.common.ellipsize(title, 120)  + (small? ' ' : ''));
+            if (small) {
+                titleElement.append(jQuery('<small/>').text(small));
+            }
+            this.header.append(titleElement);
         }
     });
 
@@ -960,19 +1029,19 @@ var Sao = {};
             return ir_model.execute('global_search',
                     [text, Sao.config.limit, Sao.main_menu_screen.model_name],
                     Sao.main_menu_screen.context)
-                .then(function(s_results) {
-                var results = [];
-                for (var i=0, len=s_results.length; i < len; i++) {
-                    results.push({
-                        'model': s_results[i][1],
-                        'model_name': s_results[i][2],
-                        'record_id': s_results[i][3],
-                        'record_name': s_results[i][4],
-                        'icon': s_results[i][5],
-                    });
-                }
-                return results;
-            }.bind(this));
+                .then(s_results => {
+                    var results = [];
+                    for (const result of s_results) {
+                        results.push({
+                            'model': result[1],
+                            'model_name': result[2],
+                            'record_id': result[3],
+                            'record_name': result[4],
+                            'icon': result[5],
+                        });
+                    }
+                    return results;
+                });
         },
         match_selected: function(item) {
             if (item.model == Sao.main_menu_screen.model_name) {
@@ -1085,18 +1154,45 @@ var Sao = {};
                 shortcut: 'f1',
                 label: Sao.i18n.gettext('Show this help'),
                 callback: function() {
-                    shortcuts_dialog();
+                    help_dialog();
+                },
+            }, {
+                shortcut: 'ctrl+f1',
+                label: Sao.i18n.gettext('Show/Hide access keys'),
+                callback: function() {
+                    jQuery('html').toggleClass('accesskey');
                 },
             },
         ];
     }
 
     jQuery(document).ready(function() {
+        var url = new URL(window.location);
+        if (url.searchParams.has('session')) {
+            var database = url.searchParams.get('database');
+            var session = {
+                login_service: url.searchParams.get('login_service'),
+                login: url.searchParams.get('login'),
+                user_id: parseInt(url.searchParams.get('user_id'), 10),
+                session: url.searchParams.get('session'),
+            };
+            if (url.searchParams.has('renew')) {
+                var renew_id = parseInt(url.searchParams.get('renew'), 10);
+                if (session.user_id !== renew_id) {
+                    window.close();
+                    return;
+                }
+            }
+            session = JSON.stringify(session);
+            localStorage.setItem('sao_session_' + database, session);
+            window.close();
+            return;
+        }
         set_shortcuts();
         try {
             Notification.requestPermission();
         } catch (e) {
-            (console.error || console.log).call(console, e, e.stack);
+            Sao.Logger.error(e.message, e.stack);
         }
         Sao.login();
     });
@@ -1122,9 +1218,10 @@ var Sao = {};
         }
     }
 
-    function shortcuts_dialog() {
-        var dialog = new Sao.Dialog(Sao.i18n.gettext('Keyboard shortcuts'),
-            'shortcut-dialog', 'm');
+    function help_dialog() {
+        var dialog = new Sao.Dialog(
+            Sao.i18n.gettext("Help"), 'help-dialog', 'md', true,
+            Sao.__version__);
         jQuery('<button>', {
             'class': 'close',
             'data-dismiss': 'modal',
@@ -1132,6 +1229,15 @@ var Sao = {};
         }).append(jQuery('<span>', {
             'aria-hidden': true,
         }).append('&times;')).prependTo(dialog.header);
+        jQuery('<a/>', {
+            'class': 'btn btn-link',
+            'href': Sao.config.doc_url,
+            'target': '_blank',
+        }).text(Sao.i18n.gettext("Documentation..."))
+            .appendTo(dialog.footer);
+        jQuery('<h4/>')
+            .text(Sao.i18n.gettext("Keyboard shortcuts"))
+            .appendTo(dialog.body);
         var row = jQuery('<div/>', {
             'class': 'row'
         }).appendTo(dialog.body);
@@ -1146,7 +1252,7 @@ var Sao = {};
             .text(Sao.i18n.gettext('Tab shortcuts')))
             .appendTo(row);
 
-        shortcuts_defs().forEach(function(definition) {
+        for (const definition of shortcuts_defs()) {
             var dt = jQuery('<dt/>').text(definition.label);
             var dd = jQuery('<dd/>').append(jQuery('<kbd>')
                 .text(definition.shortcut));
@@ -1158,7 +1264,7 @@ var Sao = {};
             }
             dt.appendTo(dest_dl);
             dd.appendTo(dest_dl);
-        });
+        }
         dialog.modal.on('hidden.bs.modal', function() {
             jQuery(this).remove();
         });
@@ -1187,9 +1293,9 @@ var Sao = {};
     // Fix Chrome bug: https://bugs.chromium.org/p/chromium/issues/detail?id=890248
     jQuery(document).on('keydown', 'textarea', function(event) {
         if (event.key === 'PageUp' || event.key === 'PageDown') {
+            event.preventDefault();
             var cursorPosition = (
                 event.key === 'PageUp' ? 0 : event.target.textLength);
-            event.preventDefault();
             event.target.setSelectionRange(cursorPosition, cursorPosition);
         }
     });
